// Compensación
// Ejemplo del péndulo invertido

clear
// Se carga la librería de funciones para el diagrama de Bode sintótico.
// L librería debe estar en el directorio de operació. 
pth=pwd()
pth=pth+"\T_d_Control_lib.sci"
exec(pth)

clc
// Cálculo del modelo lineal
// Ejemplo del péndulo invertido
// Función lin()
function [y,xp]=pendulo_2(x,u)
    M=0.5;
    m=0.2;
    B=0.1;
    Jx=0.006;
    g=9.8;
    L=0.3;
    x1p=x(2);
    x3p=x(4);
    x2p=(m^2*L^2*g*sin(x(3))*cos(x(3))-(Jx+m*L^2)*(m*L*x(4)^2*sin(x(3))-B*x(2)+u(1)))/(m^2*L^2*(cos(x(3)))^2-(Jx+m*L^2)*(M+m));
    x4p=(m*L*cos(x(3))*(-B*x(2)+m*L*x(4)^2*sin(x(3))+u(1))-m*L*g*(M+m)*sin(x(3)))/(m^2*L^2*(cos(x(3)))^2-(Jx+m*L^2)*(M+m));
    xp=[x1p;x2p;x3p;x4p];
    y=x(3);
endfunction
X0=[0;0;0;0];
U0=[0];
[A,B,C,D]=lin(pendulo_2,X0,U0);
P1=syslin("c",clean(A),clean(B),C,D)
Ga=zpk(P1)

// Compensación del péndulo Invertido
s=%s
z=%z
// Discretización
Ts=0.01             // Trecuencia de muestreo fs=100 hz
P2=dscr(Ga,Ts)
G2=zpk(P2)
G3=ss2tf(P2,"b")
zpk(G3)
// Transformación Bilineal
G4=ss2tf(bilin(G3,[Ts/2,1,-Ts/2,1]))
zpk(G4)

// Diagramas de Bode
xdel(winsid());
wr=logspace(-3,8,500);
fx=wr/2/%pi;
resp=repfreq(G4,fx);       // Respuesta en frecuencia
[DB,F]=dbphi(resp);
// Diagrama de Bode de Módulo
plot2d("ln",wr,DB,color("white"));
xlabel("Frecuencia [rad/seg]","fontsize",2)
xgrid(color("gray"))
c = color("blue");
ax2 = newaxes();
set(ax2, "filled", "off", "foreground", c, "font_color", c);
plot2d("ln",wr,DB,color("blue"));
set_my_line_styles(1,2);
ylabel(_("Módulo[dB]"),"color",[0 0 1],"fontsize",2)
set_my_line_styles(1,2);
gce().children.thickness = 2;
ax2.axes_visible(1) = "off";
ax2.y_location = "left";
title("Diagrama de Bode","fontsize",4)
// Diagrama de Bode de Fase
c = color("red");
ax3 = newaxes();
set(ax3, "filled", "off", "foreground", c, "font_color", c);
plot2d("ln",wr,F,c);
set_my_line_styles(1,2);
ylabel(_("Fase [°]"),"color",[1 0 0],"fontsize",2)
gce().children.thickness = 2;
ax3.axes_visible(1) = "off";
ax3.y_location = "right";


// Diagramas de Bode
xdel(winsid());
wr=logspace(-3,8,500);
fx=wr/2/%pi;
resp=repfreq(G4,fx);       // Respuesta en frecuencia
[DB,F]=dbphi(resp);
// Diagrama de Bode de Módulo
subplot(211)
plot2d("ln",wr,DB,color("red"));
set_my_line_styles(1,2);
xgrid(color("gray"))
title("Diagrama de Bode de Módulo ")
// Diagrama de Bode de Fase
subplot(212)
plot2d("ln",wr,F,color("blue"));
set_my_line_styles(1,2);
xgrid(color("gray"))
title("Diagrama de Bode de Fase ")


// Nyquist Logarítmico
xdel(winsid())
LogNyquist(100*G4,8,2);

// Diseño de la red
// Pone un integrador para cancelar el cero en el origen
G5=simpl(G4/s,1e-6)

// Nyquist Logarítmico
xdel(winsid())
LogNyquist(100*G5,8,2);

xdel(winsid())
bode(G5)

// Se utiliza una doble red de alelanto de 65° en 10 rad/seg
xdel(winsid())
Gc1=adelanto(65,10)
zpk(Gc1)
GH1=14*G5*Gc1^2
bode(GH1,"rad")
[MF1,Fr1]=p_margin(-GH1);
Wr1=2*%pi*Fr1
MF1
// Nyquist Logarítmico
xdel(winsid())
LogNyquist(GH1,8,2);

Gc2=simpl(GH1/G4,1e-6)
zpk(Gc2)
//Discretización
// Transformación bilineal
Gc1_z=ss2tf(bilin(Gc2,[2/Ts,-2/Ts,1,1]))
zpk(Gc1_z)


// Transferencia de lazo cerrado
//Tlc=D_z*G3/(1-G3*D_z)
Tlc=Gc1_z*G3/(1-Gc1_z*G3)
zpk(Tlc)
roots(Tlc.den)
// Respuesta transitoria
xdel(winsid())
U=[zeros(1,10),ones(1,5),zeros(1,335)]
Tt=linspace(0,length(U),350)
Yt=flts(U,Tlc)
plot2d2(Tt*Ts,Yt,color("red"));
plot2d2(Tt*Ts,U,color("blue"));
set_my_line_styles(1,2);
xgrid(color("gray"))
title("Respuesta a lazo cerrado ","fontsize",3)


